@@ -599,20 +599,16 @@ def pca(data):
     Adding the column means to the dot product of the coordinates and the
     principal components recreates the data matrix:
 
-    >>> from numpy import array, dot
+    >>> import numpy as np
     >>> from Bio.Cluster import pca
-    >>> matrix = array([[ 0.,  0.,  0.],
-    ...                 [ 1.,  0.,  0.],
-    ...                 [ 7.,  3.,  0.],
-    ...                 [ 4.,  2.,  6.]])
+    >>> matrix = np.array([[ 0.,  0.,  0.],
+    ...                    [ 1.,  0.,  0.],
+    ...                    [ 7.,  3.,  0.],
+    ...                    [ 4.,  2.,  6.]])
     >>> columnmean, coordinates, pc, _ = pca(matrix)
-    >>> m = matrix - (columnmean + dot(coordinates, pc))
-    >>> abs(m) < 1e-12
-    array([[ True,  True,  True],
-           [ True,  True,  True],
-           [ True,  True,  True],
-           [ True,  True,  True]])
-
+    >>> m = matrix - (columnmean + np.dot(coordinates, pc))
+    >>> np.all(abs(m) < 1e-12)
+    np.True_
     """
     data = __check_data(data)
     nrows, ncols = data.shape
