@@ -37,16 +37,18 @@ written solution, since the number of DTDs is rather large and their
 contents may change over time. About half the code in this parser deals
 with parsing the DTD, and the other half with the XML itself.
 """
+
 import os
 import warnings
-from collections import Counter
-from xml.parsers import expat
-from io import BytesIO
 import xml.etree.ElementTree as ET
+from collections import Counter
+from io import BytesIO
+from urllib.parse import urlparse
+from urllib.request import urlopen
+from xml.parsers import expat
 from xml.sax.saxutils import escape
 
-from urllib.request import urlopen, urlparse
-
+from Bio import StreamModeError
 
 # The following four classes are used to add a member .attributes to integers,
 # strings, lists, and dictionaries, respectively.
@@ -55,13 +57,10 @@ from urllib.request import urlopen, urlparse
 class NoneElement:
     """NCBI Entrez XML element mapped to None."""
 
-    def __init__(self, tag, attributes, key=None):
+    def __init__(self, tag, attributes, key):
         """Create a NoneElement."""
         self.tag = tag
-        if key is None:
-            self.key = tag
-        else:
-            self.key = key
+        self.key = key
         self.attributes = attributes
 
     def __eq__(self, other):
@@ -94,16 +93,15 @@ class NoneElement:
 class IntegerElement(int):
     """NCBI Entrez XML element mapped to an integer."""
 
-    def __new__(cls, value, tag, attributes, key=None):
+    def __new__(cls, value, *args, **kwargs):
         """Create an IntegerElement."""
-        self = int.__new__(cls, value)
+        return int.__new__(cls, value)
+
+    def __init__(self, value, tag, attributes, key):
+        """Initialize an IntegerElement."""
         self.tag = tag
-        if key is None:
-            self.key = tag
-        else:
-            self.key = key
         self.attributes = attributes
-        return self
+        self.key = key
 
     def __repr__(self):
         """Return a string representation of the object."""
@@ -112,22 +110,21 @@ class IntegerElement(int):
             attributes = self.attributes
         except AttributeError:
             return text
-        return "IntegerElement(%s, attributes=%r)" % (text, attributes)
+        return f"IntegerElement({text}, attributes={attributes!r})"
 
 
 class StringElement(str):
     """NCBI Entrez XML element mapped to a string."""
 
-    def __new__(cls, value, tag, attributes, key=None):
+    def __new__(cls, value, *args, **kwargs):
         """Create a StringElement."""
-        self = str.__new__(cls, value)
+        return str.__new__(cls, value)
+
+    def __init__(self, value, tag, attributes, key):
+        """Initialize a StringElement."""
         self.tag = tag
-        if key is None:
-            self.key = tag
-        else:
-            self.key = key
         self.attributes = attributes
-        return self
+        self.key = key
 
     def __repr__(self):
         """Return a string representation of the object."""
@@ -135,7 +132,7 @@ class StringElement(str):
         attributes = self.attributes
         if not attributes:
             return text
-        return "StringElement(%s, attributes=%r)" % (text, attributes)
+        return f"StringElement({text}, attributes={attributes!r})"
 
 
 class ListElement(list):
@@ -157,13 +154,14 @@ class ListElement(list):
         attributes = self.attributes
         if not attributes:
             return text
-        return "ListElement(%s, attributes=%r)" % (text, attributes)
+        return f"ListElement({text}, attributes={attributes!r})"
 
     def store(self, value):
         """Append an element to the list, checking tags."""
         key = value.key
         if self.allowed_tags is not None and key not in self.allowed_tags:
             raise ValueError("Unexpected item '%s' in list" % key)
+        del value.key
         self.append(value)
 
 
@@ -190,7 +188,7 @@ class DictionaryElement(dict):
         attributes = self.attributes
         if not attributes:
             return text
-        return "DictElement(%s, attributes=%r)" % (text, attributes)
+        return f"DictElement({text}, attributes={attributes!r})"
 
     def store(self, value):
         """Add an entry to the dictionary, checking tags."""
@@ -198,12 +196,68 @@ class DictionaryElement(dict):
         tag = value.tag
         if self.allowed_tags is not None and tag not in self.allowed_tags:
             raise ValueError("Unexpected item '%s' in dictionary" % key)
+        del value.key
         if self.repeated_tags and key in self.repeated_tags:
             self[key].append(value)
         else:
             self[key] = value
 
 
+class OrderedListElement(list):
+    """NCBI Entrez XML element mapped to a list of lists.
+
+    OrderedListElement is used to describe a list of repeating elements such as
+    A, B, C, A, B, C, A, B, C ... where each set of A, B, C forms a group. This
+    is then stored as [[A, B, C], [A, B, C], [A, B, C], ...]
+    """
+
+    def __init__(self, tag, attributes, allowed_tags, first_tag, key=None):
+        """Create an OrderedListElement."""
+        self.tag = tag
+        if key is None:
+            self.key = tag
+        else:
+            self.key = key
+        self.attributes = attributes
+        self.allowed_tags = allowed_tags
+        self.first_tag = first_tag
+
+    def __repr__(self):
+        """Return a string representation of the object."""
+        text = list.__repr__(self)
+        attributes = self.attributes
+        if not attributes:
+            return text
+        return f"OrderedListElement({text}, attributes={attributes!r})"
+
+    def store(self, value):
+        """Append an element to the list, checking tags."""
+        key = value.key
+        if self.allowed_tags is not None and key not in self.allowed_tags:
+            raise ValueError("Unexpected item '%s' in list" % key)
+        if key == self.first_tag:
+            self.append([])
+        self[-1].append(value)
+
+
+class ErrorElement(str):
+    """NCBI Entrez XML element containing an error message."""
+
+    def __new__(cls, value, *args, **kwargs):
+        """Create an ErrorElement."""
+        return str.__new__(cls, value)
+
+    def __init__(self, value, tag):
+        """Initialize an ErrorElement."""
+        self.tag = tag
+        self.key = tag
+
+    def __repr__(self):
+        """Return the error message as a string."""
+        text = str.__repr__(self)
+        return f"ErrorElement({text})"
+
+
 class NotXMLError(ValueError):
     """Failed to parse file as XML."""
 
@@ -261,7 +315,13 @@ class DataHandlerMeta(type):
 
     def __init__(cls, *args, **kwargs):
         """Initialize the class."""
-        cls._directory = None
+        from Bio import Entrez
+
+        try:
+            cls.directory = Entrez.local_cache  # use default directory for local cache
+        except PermissionError:
+            cls._directory = Entrez.local_cache  # no local cache
+        del Entrez
 
     @property
     def directory(cls):
@@ -279,13 +339,15 @@ class DataHandlerMeta(type):
             else:  # Unix/Linux/Mac
                 home = os.path.expanduser("~")
                 value = os.path.join(home, ".config", "biopython")
-        cls._directory = value
         # Create DTD local directory
-        cls.local_dtd_dir = os.path.join(cls._directory, "Bio", "Entrez", "DTDs")
+        cls.local_dtd_dir = os.path.join(value, "Bio", "Entrez", "DTDs")
         os.makedirs(cls.local_dtd_dir, exist_ok=True)
         # Create XSD local directory
-        cls.local_xsd_dir = os.path.join(cls._directory, "Bio", "Entrez", "XSDs")
+        cls.local_xsd_dir = os.path.join(value, "Bio", "Entrez", "XSDs")
         os.makedirs(cls.local_xsd_dir, exist_ok=True)
+        # Save the directory name after creating the DTD and XSD local
+        # directories to ensure there was no PermissionError.
+        cls._directory = value
 
 
 class DataHandler(metaclass=DataHandlerMeta):
@@ -295,12 +357,12 @@ class DataHandler(metaclass=DataHandlerMeta):
 
     global_dtd_dir = os.path.join(Entrez.__path__[0], "DTDs")
     global_xsd_dir = os.path.join(Entrez.__path__[0], "XSDs")
-    local_dtd_dir = ""
-    local_xsd_dir = ""
+    local_dtd_dir = None
+    local_xsd_dir = None
 
     del Entrez
 
-    def __init__(self, validate, escape):
+    def __init__(self, validate, escape, ignore_errors):
         """Create a DataHandler object."""
         self.dtd_urls = []
         self.element = None
@@ -308,12 +370,12 @@ class DataHandler(metaclass=DataHandlerMeta):
         self.data = []
         self.attributes = None
         self.allowed_tags = None
+        self.constructors = {}
         self.strings = {}
-        self.lists = {}
-        self.dictionaries = {}
         self.items = set()
         self.errors = set()
         self.validating = validate
+        self.ignore_errors = ignore_errors
         self.parser = expat.ParserCreate(namespace_separator=" ")
         self.parser.SetParamEntityParsing(expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
         self.parser.XmlDeclHandler = self.xmlDeclHandler
@@ -325,14 +387,22 @@ class DataHandler(metaclass=DataHandlerMeta):
         else:
             self.characterDataHandler = self.characterDataHandlerRaw
 
-    def read(self, handle):
-        """Set up the parser and let it parse the XML results."""
+    def read(self, source):
+        """Set up the parser and let it read the XML results."""
         # Expat's parser.ParseFile function only accepts binary data;
         # see also the comment below for Entrez.parse.
-        if handle.read(0) != b"":
+        try:
+            stream = open(source, "rb")
+        except TypeError:  # not a path, assume we received a stream
+            if source.read(0) != b"":
+                raise StreamModeError(
+                    "the XML file must be opened in binary mode."
+                ) from None
+            stream = source
+        if stream.read(0) != b"":
             raise TypeError("file should be opened in binary mode")
         try:
-            self.parser.ParseFile(handle)
+            self.parser.ParseFile(stream)
         except expat.ExpatError as e:
             if self.parser.StartElementHandler:
                 # We saw the initial <!xml declaration, so we can be sure that
@@ -343,8 +413,11 @@ class DataHandler(metaclass=DataHandlerMeta):
                 # We have not seen the initial <!xml declaration, so probably
                 # the input data is not in XML format.
                 raise NotXMLError(e) from None
+        finally:
+            if stream is not source:
+                stream.close()
         try:
-            return self.record
+            record = self.record
         except AttributeError:
             if self.parser.StartElementHandler:
                 # We saw the initial <!xml declaration, and expat didn't notice
@@ -359,72 +432,87 @@ class DataHandler(metaclass=DataHandlerMeta):
                 # We did not see the initial <!xml declaration, so probably
                 # the input data is not in XML format.
                 raise NotXMLError("XML declaration not found") from None
+        else:
+            del record.key
+            return record
 
-    def parse(self, handle):
-        """Parse the XML in the given file handle."""
-        # The handle should have been opened in binary mode; data read from
-        # the handle are then bytes. Expat will pick up the encoding from the
-        # XML declaration (or assume UTF-8 if it is missing), and use this
-        # encoding to convert the binary data to a string before giving it to
-        # characterDataHandler.
+    def parse(self, source):
+        """Set up the parser and let it read the XML results."""
+        # The source must be a filename, or a file-like object opened in binary
+        # mode. Data read from the file or file-like object as bytes. Expat will
+        # pick up the encoding from the XML declaration (or assume UTF-8 if it
+        # is missing), and use this encoding to convert the binary data to a
+        # string before giving it to characterDataHandler.
         # While parser.ParseFile only accepts binary data, parser.Parse accepts
         # both binary data and strings. However, a file in text mode may have
         # been opened with an encoding different from the encoding specified in
         # the XML declaration at the top of the file. If so, the data in the
         # file will have been decoded with an incorrect encoding. To avoid
         # this, and to be consistent with parser.ParseFile (which is used in
-        # the Entrez.read function above), we require the handle to be in
+        # the Entrez.read function above), we require the source data to be in
         # binary mode here as well.
-        if handle.read(0) != b"":
+        try:
+            stream = open(source, "rb")
+        except TypeError:  # not a path, assume we received a stream
+            if source.read(0) != b"":
+                raise StreamModeError(
+                    "the XML file must be opened in binary mode."
+                ) from None
+            stream = source
+        if stream.read(0) != b"":
             raise TypeError("file should be opened in binary mode")
         BLOCK = 1024
-        while True:
-            # Read in another block of data from the file.
-            data = handle.read(BLOCK)
-            try:
+        try:
+            while True:
+                # Read in another block of data from the file.
+                data = stream.read(BLOCK)
                 self.parser.Parse(data, False)
-            except expat.ExpatError as e:
-                if self.parser.StartElementHandler:
-                    # We saw the initial <!xml declaration, so we can be sure
-                    # that we are parsing XML data. Most likely, the XML file
-                    # is corrupted.
-                    raise CorruptedXMLError(e) from None
-                else:
-                    # We have not seen the initial <!xml declaration, so
-                    # probably the input data is not in XML format.
-                    raise NotXMLError(e) from None
-            try:
-                records = self.record
-            except AttributeError:
-                if self.parser.StartElementHandler:
-                    # We saw the initial <!xml declaration, and expat
-                    # didn't notice any errors, so self.record should be
-                    # defined. If not, this is a bug.
+                try:
+                    records = self.record
+                except AttributeError:
+                    if self.parser.StartElementHandler:
+                        # We saw the initial <!xml declaration, and expat
+                        # didn't notice any errors, so self.record should be
+                        # defined. If not, this is a bug.
 
-                    raise RuntimeError(
-                        "Failed to parse the XML file correctly, possibly due to a "
-                        "bug in Bio.Entrez. Please contact the Biopython "
-                        "developers via the mailing list or GitHub for assistance."
-                    ) from None
-                else:
-                    # We did not see the initial <!xml declaration, so
-                    # probably the input data is not in XML format.
-                    raise NotXMLError("XML declaration not found") from None
+                        raise RuntimeError(
+                            "Failed to parse the XML file correctly, possibly due to a "
+                            "bug in Bio.Entrez. Please contact the Biopython "
+                            "developers via the mailing list or GitHub for assistance."
+                        ) from None
+                    else:
+                        # We did not see the initial <!xml declaration, so
+                        # probably the input data is not in XML format.
+                        raise NotXMLError("XML declaration not found") from None
 
-            if not isinstance(records, list):
-                raise ValueError(
-                    "The XML file does not represent a list. Please use Entrez.read "
-                    "instead of Entrez.parse"
-                )
+                if not isinstance(records, list):
+                    raise ValueError(
+                        "The XML file does not represent a list. Please use "
+                        "Entrez.read instead of Entrez.parse."
+                    )
 
-            if not data:
-                break
+                if not data:
+                    break
 
-            while len(records) >= 2:
-                # Then the first record is finished, while the second record
-                # is still a work in progress.
-                record = records.pop(0)
-                yield record
+                while len(records) >= 2:
+                    # Then the first record is finished, while the second record
+                    # is still a work in progress.
+                    record = records.pop(0)
+                    yield record
+
+        except expat.ExpatError as e:
+            if self.parser.StartElementHandler:
+                # We saw the initial <!xml declaration, so we can be sure
+                # that we are parsing XML data. Most likely, the XML file
+                # is corrupted.
+                raise CorruptedXMLError(e) from None
+            else:
+                # We have not seen the initial <!xml declaration, so
+                # probably the input data is not in XML format.
+                raise NotXMLError(e) from None
+        finally:
+            if stream is not source:
+                stream.close()
 
         # We have reached the end of the XML file
         self.parser = None
@@ -467,8 +555,10 @@ class DataHandler(metaclass=DataHandlerMeta):
                 assert uri == "http://www.w3.org/1998/Math/MathML"
             elif prefix == "xlink":
                 assert uri == "http://www.w3.org/1999/xlink"
+            elif prefix == "ali":
+                assert uri.rstrip("/") == "http://www.niso.org/schemas/ali/1.0"
             else:
-                raise ValueError("Unknown prefix '%s' with uri '%s'" % (prefix, uri))
+                raise ValueError(f"Unknown prefix '{prefix}' with uri '{uri}'")
             self.namespace_level[prefix] += 1
             self.namespace_prefix[uri] = prefix
 
@@ -506,6 +596,15 @@ class DataHandler(metaclass=DataHandlerMeta):
 
     def startElementHandler(self, tag, attrs):
         """Handle start of an XML element."""
+        prefix = None
+        if self.namespace_prefix:
+            try:
+                uri, name = tag.split()
+            except ValueError:
+                pass
+            else:
+                prefix = self.namespace_prefix[uri]
+                tag = f"{prefix}:{name}"
         if tag in self.items:
             assert tag == "Item"
             name = attrs["Name"]
@@ -584,22 +683,9 @@ class DataHandler(metaclass=DataHandlerMeta):
             self.allowed_tags = self.strings[tag]
             assert self.attributes is None
             self.attributes = attrs
-        elif tag in self.dictionaries:
-            allowed_tags, repeated_tags = self.dictionaries[tag]
-            element = DictionaryElement(tag, attrs, allowed_tags, repeated_tags)
-            parent = self.element
-            element.parent = parent
-            # For consistency with lists below, store the element here
-            if parent is None:
-                self.record = element
-            else:
-                parent.store(element)
-            self.element = element
-            self.parser.EndElementHandler = self.endElementHandler
-            self.parser.CharacterDataHandler = self.skipCharacterDataHandler
-        elif tag in self.lists:
-            allowed_tags = self.lists[tag]
-            element = ListElement(tag, attrs, allowed_tags)
+        elif tag in self.constructors:
+            cls, allowed_tags = self.constructors[tag]
+            element = cls(tag, attrs, *allowed_tags)
             parent = self.element
             element.parent = parent
             if parent is None:
@@ -612,14 +698,28 @@ class DataHandler(metaclass=DataHandlerMeta):
             self.parser.CharacterDataHandler = self.skipCharacterDataHandler
         else:
             # Element not found in DTD
-            if self.validating:
+            if tag == "processing-meta":
+                terms = []
+                dtd_version = "1.3"
+                if attrs["tagset-family"] == "jats":
+                    terms.append("JATS")
+                if attrs["base-tagset"] == "archiving":
+                    term = "archivearticle" + dtd_version.replace(".", "-")
+                    terms.append(term)
+                if attrs.get("mathml-version") == "3.0":
+                    terms.append("mathml3")
+                basename = "-".join(terms)
+                url = f"https://{attrs['tagset-family']}.nlm.nih.gov/{attrs['base-tagset']}/{dtd_version}/{basename}.dtd"
+                self.xmlDeclHandler(None, None, None)
+                self.externalEntityRefHandler(None, None, url, None)
+                # remainder will be ignored and will not be stored in the record
+            elif self.validating:
                 raise ValidationError(tag)
-            else:
-                # this will not be stored in the record
-                self.parser.StartElementHandler = self.startSkipElementHandler
-                self.parser.EndElementHandler = self.endSkipElementHandler
-                self.parser.CharacterDataHandler = self.skipCharacterDataHandler
-                self.level = 1
+            # this will not be stored in the record
+            self.parser.StartElementHandler = self.startSkipElementHandler
+            self.parser.EndElementHandler = self.endSkipElementHandler
+            self.parser.CharacterDataHandler = self.skipCharacterDataHandler
+            self.level = 1
 
     def startRawElementHandler(self, name, attrs):
         """Handle start of an XML raw element."""
@@ -635,14 +735,14 @@ class DataHandler(metaclass=DataHandlerMeta):
                 if self.namespace_level[prefix] == 1:
                     attrs = {"xmlns": uri}
         if prefix:
-            key = "%s:%s" % (prefix, name)
+            key = f"{prefix}:{name}"
         else:
             key = name
         # self.allowed_tags is ignored for now. Anyway we know what to do
         # with this tag.
         tag = "<%s" % name
         for key, value in attrs.items():
-            tag += ' %s="%s"' % (key, value)
+            tag += f' {key}="{value}"'
         tag += ">"
         self.data.append(tag)
         self.parser.EndElementHandler = self.endRawElementHandler
@@ -659,17 +759,25 @@ class DataHandler(metaclass=DataHandlerMeta):
             self.parser.StartElementHandler = self.startElementHandler
             self.parser.EndElementHandler = self.endElementHandler
             self.parser.CharacterDataHandler = self.skipCharacterDataHandler
-        value = "".join(self.data)
+        data = "".join(self.data)
         self.data = []
         attributes = self.attributes
         self.attributes = None
+        if self.namespace_prefix:
+            try:
+                uri, name = tag.split()
+            except ValueError:
+                pass
+            else:
+                prefix = self.namespace_prefix[uri]
+                tag = f"{prefix}:{name}"
         if tag in self.items:
             assert tag == "Item"
             key = attributes["Name"]
             del attributes["Name"]
         else:
             key = tag
-        value = StringElement(value, tag, attributes, key)
+        value = StringElement(data, tag, attributes, key)
         if element is None:
             self.record = element
         else:
@@ -677,7 +785,7 @@ class DataHandler(metaclass=DataHandlerMeta):
         self.allowed_tags = None
 
     def endRawElementHandler(self, name):
-        """Handle start of an XML raw element."""
+        """Handle end of an XML raw element."""
         self.level -= 1
         if self.level == 0:
             self.parser.EndElementHandler = self.endStringElementHandler
@@ -690,22 +798,30 @@ class DataHandler(metaclass=DataHandlerMeta):
         self.data.append(tag)
 
     def endSkipElementHandler(self, name):
-        """Handle start of an XML skip element."""
+        """Handle end of an XML skip element."""
         self.level -= 1
         if self.level == 0:
             self.parser.StartElementHandler = self.startElementHandler
             self.parser.EndElementHandler = self.endElementHandler
 
-    def endErrorElementHandler(self, name):
-        """Handle start of an XML error element."""
-        if self.data:
-            # error found:
-            value = "".join(self.data)
-            raise RuntimeError(value)
-        # no error found:
-        if self.element is not None:
+    def endErrorElementHandler(self, tag):
+        """Handle end of an XML error element."""
+        element = self.element
+        if element is not None:
+            self.parser.StartElementHandler = self.startElementHandler
             self.parser.EndElementHandler = self.endElementHandler
             self.parser.CharacterDataHandler = self.skipCharacterDataHandler
+        data = "".join(self.data)
+        if data == "":
+            return
+        if self.ignore_errors is False:
+            raise RuntimeError(data)
+        self.data = []
+        value = ErrorElement(data, tag)
+        if element is None:
+            self.record = element
+        else:
+            element.store(value)
 
     def endElementHandler(self, name):
         """Handle end of an XML element."""
@@ -787,11 +903,13 @@ class DataHandler(metaclass=DataHandlerMeta):
             allowed_tags = frozenset(keys)
             if len(keys) == 1 and keys == multiple:
                 assert not isSimpleContent
-                self.lists[name] = allowed_tags
+                args = (allowed_tags,)
+                self.constructors[name] = (ListElement, args)
             elif len(keys) >= 1:
                 assert not isSimpleContent
                 repeated_tags = frozenset(multiple)
-                self.dictionaries[name] = (allowed_tags, repeated_tags)
+                args = (allowed_tags, repeated_tags)
+                self.constructors[name] = (DictionaryElement, args)
             else:
                 self.strings[name] = allowed_tags
 
@@ -835,16 +953,31 @@ class DataHandler(metaclass=DataHandlerMeta):
                 allowed_tags = frozenset()
             self.strings[name] = allowed_tags
             return
+        # Children can be anything; use a dictionary-type element
+        if model == (expat.model.XML_CTYPE_ANY, expat.model.XML_CQUANT_NONE, None, ()):
+            allowed_tags = None
+            repeated_tags = None
+            args = (allowed_tags, repeated_tags)
+            self.constructors[name] = (DictionaryElement, args)
+            return
         # List-type elements
         if model[0] in (
             expat.model.XML_CTYPE_CHOICE,
             expat.model.XML_CTYPE_SEQ,
         ) and model[1] in (expat.model.XML_CQUANT_PLUS, expat.model.XML_CQUANT_REP):
             children = model[3]
-            if model[0] == expat.model.XML_CTYPE_SEQ:
-                assert len(children) == 1
             allowed_tags = frozenset(child[2] for child in children)
-            self.lists[name] = allowed_tags
+            if model[0] == expat.model.XML_CTYPE_SEQ:
+                if len(children) > 1:
+                    assert model[1] == expat.model.XML_CQUANT_PLUS
+                    first_child = children[0]
+                    assert first_child[1] == expat.model.XML_CQUANT_NONE
+                    first_tag = first_child[2]
+                    args = allowed_tags, first_tag
+                    self.constructors[name] = (OrderedListElement, args)
+                    return
+                assert len(children) == 1
+            self.constructors[name] = (ListElement, (allowed_tags,))
             return
         # This is the tricky case. Check which keys can occur multiple
         # times. If only one key is possible, and it can occur multiple
@@ -855,9 +988,9 @@ class DataHandler(metaclass=DataHandlerMeta):
         # only once, and which can occur multiple times.
         single = []
         multiple = []
+        errors = []
         # The 'count' function is called recursively to make sure all the
-        # children in this model are counted. Error keys are ignored;
-        # they raise an exception in Python.
+        # children in this model are counted.
 
         def count(model):
             quantifier, key, children = model[1:]
@@ -871,7 +1004,9 @@ class DataHandler(metaclass=DataHandlerMeta):
                 else:
                     for child in children:
                         count(child)
-            elif key.upper() != "ERROR":
+            elif key.upper() == "ERROR":
+                errors.append(key)
+            else:
                 if quantifier in (
                     expat.model.XML_CQUANT_NONE,
                     expat.model.XML_CQUANT_OPT,
@@ -885,22 +1020,24 @@ class DataHandler(metaclass=DataHandlerMeta):
 
         count(model)
         if len(single) == 0 and len(multiple) == 1:
-            allowed_tags = frozenset(multiple)
-            self.lists[name] = allowed_tags
+            allowed_tags = frozenset(multiple + errors)
+            self.constructors[name] = (ListElement, (allowed_tags,))
         else:
-            allowed_tags = frozenset(single + multiple)
+            allowed_tags = frozenset(single + multiple + errors)
             repeated_tags = frozenset(multiple)
-            self.dictionaries[name] = (allowed_tags, repeated_tags)
+            args = (allowed_tags, repeated_tags)
+            self.constructors[name] = (DictionaryElement, args)
 
     def open_dtd_file(self, filename):
         """Open specified DTD file."""
-        path = os.path.join(DataHandler.local_dtd_dir, filename)
-        try:
-            handle = open(path, "rb")
-        except FileNotFoundError:
-            pass
-        else:
-            return handle
+        if DataHandler.local_dtd_dir is not None:
+            path = os.path.join(DataHandler.local_dtd_dir, filename)
+            try:
+                handle = open(path, "rb")
+            except FileNotFoundError:
+                pass
+            else:
+                return handle
         path = os.path.join(DataHandler.global_dtd_dir, filename)
         try:
             handle = open(path, "rb")
@@ -912,13 +1049,14 @@ class DataHandler(metaclass=DataHandlerMeta):
 
     def open_xsd_file(self, filename):
         """Open specified XSD file."""
-        path = os.path.join(DataHandler.local_xsd_dir, filename)
-        try:
-            handle = open(path, "rb")
-        except FileNotFoundError:
-            pass
-        else:
-            return handle
+        if DataHandler.local_xsd_dir is not None:
+            path = os.path.join(DataHandler.local_xsd_dir, filename)
+            try:
+                handle = open(path, "rb")
+            except FileNotFoundError:
+                pass
+            else:
+                return handle
         path = os.path.join(DataHandler.global_xsd_dir, filename)
         try:
             handle = open(path, "rb")
@@ -930,22 +1068,26 @@ class DataHandler(metaclass=DataHandlerMeta):
 
     def save_dtd_file(self, filename, text):
         """Save DTD file to cache."""
+        if DataHandler.local_dtd_dir is None:
+            return
         path = os.path.join(DataHandler.local_dtd_dir, filename)
         try:
             handle = open(path, "wb")
         except OSError:
-            warnings.warn("Failed to save %s at %s" % (filename, path))
+            warnings.warn(f"Failed to save {filename} at {path}")
         else:
             handle.write(text)
             handle.close()
 
     def save_xsd_file(self, filename, text):
         """Save XSD file to cache."""
+        if DataHandler.local_xsd_dir is None:
+            return
         path = os.path.join(DataHandler.local_xsd_dir, filename)
         try:
             handle = open(path, "wb")
         except OSError:
-            warnings.warn("Failed to save %s at %s" % (filename, path))
+            warnings.warn(f"Failed to save {filename} at {path}")
         else:
             handle.write(text)
             handle.close()
@@ -988,9 +1130,7 @@ class DataHandler(metaclass=DataHandlerMeta):
             try:
                 handle = urlopen(url)
             except OSError:
-                raise RuntimeError(
-                    "Failed to access %s at %s" % (filename, url)
-                ) from None
+                raise RuntimeError(f"Failed to access {filename} at {url}") from None
             text = handle.read()
             handle.close()
             self.save_dtd_file(filename, text)
