@@ -16,10 +16,10 @@ Examples
 6
 >>> print(X.count_amino_acids()['E'])
 12
->>> print("%0.2f" % X.get_amino_acids_percent()['A'])
-0.04
->>> print("%0.2f" % X.get_amino_acids_percent()['L'])
-0.12
+>>> print("%0.2f" % X.amino_acids_percent['A'])
+3.95
+>>> print("%0.2f" % X.amino_acids_percent['L'])
+11.84
 >>> print("%0.2f" % X.molecular_weight())
 17103.16
 >>> print("%0.2f" % X.aromaticity())
@@ -49,13 +49,16 @@ Other public methods are:
 
 """
 
-
+import functools
 import sys
-from Bio.SeqUtils import ProtParamData  # Local
-from Bio.SeqUtils import IsoelectricPoint  # Local
-from Bio.Seq import Seq
+import warnings
+
+from Bio import BiopythonDeprecationWarning
 from Bio.Data import IUPACData
+from Bio.Seq import Seq
+from Bio.SeqUtils import IsoelectricPoint  # Local
 from Bio.SeqUtils import molecular_weight
+from Bio.SeqUtils import ProtParamData  # Local
 
 
 class ProteinAnalysis:
@@ -77,7 +80,6 @@ class ProteinAnalysis:
         """Initialize the class."""
         self.sequence = prot_sequence.upper()
         self.amino_acids_content = None
-        self.amino_acids_percent = None
         self.length = len(self.sequence)
         self.monoisotopic = monoisotopic
 
@@ -100,24 +102,32 @@ class ProteinAnalysis:
         return self.amino_acids_content
 
     def get_amino_acids_percent(self):
-        """Calculate the amino acid content in percentages.
+        """Included for backwards compatibility (DEPRECATED)."""
+        warnings.warn(
+            "The get_amino_acids_percent method has been deprecated "
+            "and will likely be removed from Biopython in the near "
+            "future. Please use the amino_acids_percent attribute instead.",
+            BiopythonDeprecationWarning,
+        )
+
+        return {aa: percent / 100 for aa, percent in self.amino_acids_percent.items()}
+
+    @functools.cached_property
+    def amino_acids_percent(self):
+        """Get the amino acid content in percentages.
 
         The same as count_amino_acids only returns the Number in percentage of
         entire sequence. Returns a dictionary of {AminoAcid:percentage}.
 
-        The return value is cached in self.amino_acids_percent.
-
-        input is the dictionary self.amino_acids_content.
-        output is a dictionary with amino acids as keys.
+        Unlike the deprecated get_amino_acids_percent method, this attribute
+        returns percentages in the range 0-100.
         """
-        if self.amino_acids_percent is None:
-            aa_counts = self.count_amino_acids()
+        aa_counts = self.count_amino_acids()
+        percentages = {
+            aa: (count * 100 / self.length) for aa, count in aa_counts.items()
+        }
 
-            percentages = {aa: count / self.length for aa, count in aa_counts.items()}
-
-            self.amino_acids_percent = percentages
-
-        return self.amino_acids_percent
+        return percentages
 
     def molecular_weight(self):
         """Calculate MW from Protein sequence."""
@@ -132,9 +142,9 @@ class ProteinAnalysis:
         It is simply the relative frequency of Phe+Trp+Tyr.
         """
         aromatic_aas = "YWF"
-        aa_percentages = self.get_amino_acids_percent()
+        aa_percentages = self.amino_acids_percent
 
-        aromaticity = sum(aa_percentages[aa] for aa in aromatic_aas)
+        aromaticity = sum(aa_percentages[aa] / 100 for aa in aromatic_aas)
 
         return aromaticity
 
@@ -329,11 +339,11 @@ class ProteinAnalysis:
 
         Returns a tuple of three floats (Helix, Turn, Sheet).
         """
-        aa_percentages = self.get_amino_acids_percent()
+        aa_percentages = self.amino_acids_percent
 
-        helix = sum(aa_percentages[r] for r in "EMALK")
-        turn = sum(aa_percentages[r] for r in "NPGSD")
-        sheet = sum(aa_percentages[r] for r in "VIYFWLT")
+        helix = sum(aa_percentages[r] / 100 for r in "EMALK")
+        turn = sum(aa_percentages[r] / 100 for r in "NPGSD")
+        sheet = sum(aa_percentages[r] / 100 for r in "VIYFWLT")
 
         return helix, turn, sheet
 
