@@ -5,19 +5,13 @@
 
 """Output of PDB files."""
 
-
+import os
 import warnings
 
-# Exceptions and Warnings
 from Bio import BiopythonWarning
-from Bio.PDB.PDBExceptions import PDBIOException
-
-# To allow saving of chains, residues, etc..
-from Bio.PDB.StructureBuilder import StructureBuilder
-
-# Allowed Elements
 from Bio.Data.IUPACData import atom_weights
-
+from Bio.PDB.PDBExceptions import PDBIOException
+from Bio.PDB.StructureBuilder import StructureBuilder
 
 _ATOM_FORMAT_STRING = (
     "%s%5i %-4s%c%3s %c%4i%c   %8.3f%8.3f%8.3f%s%6.2f      %4s%2s%2s\n"
@@ -294,6 +288,32 @@ class PDBIO(StructureIO):
 
             return _PQR_ATOM_FORMAT_STRING % args
 
+    @staticmethod
+    def _revert_write(handle, truncate_to=None, delete_file=False):
+        """Revert data written to file by removing the file or truncating it.
+
+        This method is used when the writer throws an exception, to avoid
+        writing incomplete files that might confuse users or workflows.
+        """
+        if delete_file:
+            try:
+                handle.close()
+                os.remove(handle.name)
+            except OSError as err:
+                # Windows can be finnicky with closing
+                # file and deleting them, raising PermissionError
+                pass
+        elif truncate_to is not None:
+            # If the user gave a file handle, seek back to the
+            # starting position and truncate the file from there
+            # on. Note that the truncation depends on how we
+            # opened the file, but we assume the file was opened
+            # for writing/appending anyway.
+            handle.seek(truncate_to)
+            handle.truncate()
+        else:
+            raise Exception("One of 'truncate_to' or 'delete_file' must be provided")
+
     # Public methods
     def save(self, file, select=_select, write_end=True, preserve_atom_numbering=False):
         """Save structure to a file.
@@ -321,6 +341,7 @@ class PDBIO(StructureIO):
             fhandle = open(file, "w")
         else:
             # filehandle, I hope :-)
+            fd_position = file.tell()
             fhandle = file
 
         get_atom_line = self._get_atom_line
@@ -348,8 +369,13 @@ class PDBIO(StructureIO):
                     continue
                 chain_id = chain.id
                 if len(chain_id) > 1:
-                    e = f"Chain id ('{chain_id}') exceeds PDB format limit."
-                    raise PDBIOException(e)
+                    if isinstance(file, str):
+                        self._revert_write(fhandle, delete_file=True)
+                    else:
+                        self._revert_write(fhandle, truncate_to=fd_position)
+                    raise PDBIOException(
+                        f"Chain id ('{chain_id}') exceeds PDB format limit."
+                    )
 
                 # necessary for TER
                 # do not write TER if no residues were written
@@ -364,8 +390,14 @@ class PDBIO(StructureIO):
                     segid = residue.segid
                     resid = residue.id[1]
                     if resid > 9999:
-                        e = f"Residue number ('{resid}') exceeds PDB format limit."
-                        raise PDBIOException(e)
+                        if isinstance(file, str):
+                            self._revert_write(fhandle, delete_file=True)
+                        else:
+                            self._revert_write(fhandle, truncate_to=fd_position)
+
+                        raise PDBIOException(
+                            f"Residue number ('{resid}') exceeds PDB format limit."
+                        )
 
                     for atom in residue.get_unpacked_list():
                         if not select.accept_atom(atom):
@@ -387,9 +419,14 @@ class PDBIO(StructureIO):
                                 chain_id,
                             )
                         except Exception as err:
+                            if isinstance(file, str):
+                                self._revert_write(fhandle, delete_file=True)
+                            else:
+                                self._revert_write(fhandle, truncate_to=fd_position)
+
                             # catch and re-raise with more information
                             raise PDBIOException(
-                                f"Error when writing atom {atom.full_id}"
+                                f"Error when writing atom {atom.full_id}: {err}"
                             ) from err
                         else:
                             fhandle.write(s)
